@@ -28,7 +28,6 @@
 #include <ctype.h>
 #include <string.h>
 #include <time.h>
-#include <malloc.h>
 #include "reedsol.h"
 #include "iec16022ecc200.h"
 
@@ -72,9 +71,9 @@ static const struct ecc200matrix_s {
 };
 
  // simple checked response malloc
-static void *safemalloc(int n)
+static void *safemalloc(int n, int s)
 {
-	void *p = malloc(n);
+	void *p = calloc(n, s);
 	if (!p) {
 		fprintf(stderr, "Malloc(%d) failed\n", n);
 		exit(1);
@@ -161,11 +160,8 @@ static void ecc200placementcornerD(int *array, int NR, int NC, int p)
 // Annex M placement alorithm main function
 static void ecc200placement(int *array, int NR, int NC)
 {
+	// array must be cleared by caller
 	int r, c, p;
-	// invalidate
-	for (r = 0; r < NR; r++)
-		for (c = 0; c < NC; c++)
-			array[r * NC + c] = 0;
 	// start
 	p = 1;
 	r = 4;
@@ -249,8 +245,8 @@ static char ecc200encode(unsigned char *t, int tl, const unsigned char *s, int s
 	// do the encoding
 	while (sp < sl && tp < tl) {
 		char newenc = enc;	// suggest new encoding
-		if (tl - tp <= 1 && (enc == 'c' || enc == 't') || tl - tp <= 2
-		    && enc == 'x')
+		if (tl - tp <= 1 && (enc == 'c' || enc == 't' || enc == 'x') || tl - tp <= 2
+		    && enc == 'e')
 			enc = 'a';	// auto revert to ASCII
 		newenc = tolower(encoding[sp]);
 		switch (newenc) {	// encode character
@@ -260,9 +256,9 @@ static char ecc200encode(unsigned char *t, int tl, const unsigned char *s, int s
 			{
 				char out[6];
 				int p = 0;
-				const char *e,
-				    *s2 = "!\"#$%&'()*+,-./:;<=>?@[\\]^_",
-				    *s3 = 0;
+				const char *e = " 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ\r*>"; // for X12
+				const char *s2 = "!\"#$%&'()*+,-./:;<=>?@[\\]^_";
+				const char *s3 = 0;
 				if (newenc == 'c') {
 					e = " 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 					s3 = "`abcdefghijklmnopqrstuvwxyz{|}~\177";
@@ -271,8 +267,6 @@ static char ecc200encode(unsigned char *t, int tl, const unsigned char *s, int s
 					e = " 0123456789abcdefghijklmnopqrstuvwxyz";
 					s3 = "`ABCDEFGHIJKLMNOPQRSTUVWXYZ{|}~\177";
 				}
-				if (newenc == 'x')
-					e = " 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ\r*>";
 				do {
 					unsigned char c = s[sp++];
 					char *w;
@@ -524,7 +518,7 @@ static char *encmake(int l, const unsigned char *s, int *lenp, char exact)
 	if (lenp)
 		*lenp = 0;
 	if (!l) {
-		encoding = safemalloc(1);
+		encoding = safemalloc(1, 1);
 		*encoding = 0;
 		return encoding;	// no length
 	}
@@ -818,7 +812,7 @@ static char *encmake(int l, const unsigned char *s, int *lenp, char exact)
 		 * fprintf (stderr, "\n");
 		 */
 	}
-	encoding = safemalloc(l + 1);
+	encoding = safemalloc(1, l + 1);
 	p = 0;
 	{
 		int cur = E_ASCII;	// starts ASCII
@@ -958,16 +952,15 @@ unsigned char *iec16022ecc200f(int *Wptr, int *Hptr, char **encodingptr,
 	// ecc code
 	ecc200(binary, matrix->bytes, matrix->datablock, matrix->rsblock);
 	{			// placement
+		int border_offset_y;
 		int x, y, NC, NR, *places;
 		NC = W - 2 * (W / matrix->FW);
 		NR = H - 2 * (H / matrix->FH);
-		places = safemalloc(NC * NR * sizeof(int));
+		places = safemalloc(NC, NR * sizeof(int));
 		ecc200placement(places, NR, NC);
-		grid = safemalloc(W * H);
-		memset(grid, 0, W * H);
+		grid = safemalloc(H, W + 16); // extra padding to simplify some operations
 		for (y = 0; y < H; y += matrix->FH) {
-			for (x = 0; x < W; x++)
-				grid[y * W + x] = 1;
+			memset(grid + y * W, 1, W);
 			for (x = 0; x < W; x += 2)
 				grid[(y + matrix->FH - 1) * W + x] = 1;
 		}
@@ -977,16 +970,20 @@ unsigned char *iec16022ecc200f(int *Wptr, int *Hptr, char **encodingptr,
 			for (y = 0; y < H; y += 2)
 				grid[y * W + x + matrix->FW - 1] = 1;
 		}
+		border_offset_y = -1;
 		for (y = 0; y < NR; y++) {
+			int border_offset_x = -1;
+			// jump over the 2 border pixels
+			if (y % (matrix->FH - 2) == 0) border_offset_y += 2;
 			for (x = 0; x < NC; x++) {
 				int v = places[(NR - y - 1) * NC + x];
+				// jump over the 2 border pixels
+				if (x % (matrix->FW - 2) == 0) border_offset_x += 2;
 				//fprintf (stderr, "%4d", v);
 				if (v == 1 || v > 7
 				    && (binary[(v >> 3) - 1] & (1 << (v & 7))))
-					grid[(1 + y +
-					      2 * (y / (matrix->FH - 2))) * W +
-					     1 + x +
-					     2 * (x / (matrix->FW - 2))] = 1;
+					grid[(y + border_offset_y) * W +
+					     x + border_offset_x] = 1;
 			}
 			//fprintf (stderr, "\n");
 		}
